-------------------------------------------------------------------
-------------------------------------------------------------------
-- Title:           Holic Bot                                    --
-- Author:          Elverion                                     --
-- Date:            Oct. 13, 2008                                --
-- Requirements:    MicroMacro v0.98                             --
-- License:         Public Domain                                --
-------------------------------------------------------------------
-------------------------------------------------------------------
-- This script is public domain. You may use it in any way you   --
-- find appropriate without restriction. This includes, but is   --
-- not limited to, modification and distribution, whether it be  --
-- commercial or not.                                            --
-------------------------------------------------------------------
-------------------------------------------------------------------


function getSelectedWindow()
  local windows =  findWindowList(window_name);

  if( #windows == 0 ) then
    error("No Holic windows found. You must be logged into your character first!\n", 0);
  end

  if( #windows == 1 ) then
    -- only 1 window found, so return it.
    printf("Window found: %s\n", getWindowName(windows[1]));
    return windows[1];
  else
    -- multiple windows found, ask them to select the window and press INSERT.
    printf("Multiple Holic windows were found. Please select the Holic window you wish to bot, and press INSERT on your keyboard.\n\n");
    printf("Waiting...\n");

    while( not keyPressed(key.VK_INSERT) ) do
      yrest(1);
    end

    return foregroundWindow();
  end
end


-- Update player and other general information
function updateVars()
  function __err()
    error("Error accessing memory from Holic.\n", 0);
  end

  local success;
  PlayerHP,success = memoryReadIntPtr(proc, char_baseptr, charHP_offset);
  if( not success ) then __err(); return; end;

  PlayerMaxHP,success = memoryReadIntPtr(proc, char_baseptr, charMaxHP_offset);
  if( not success ) then __err(); return; end;

  PlayerMP,success = memoryReadIntPtr(proc, char_baseptr, charMP_offset);
  if( not success ) then __err(); return; end;

  PlayerMaxMP,success = memoryReadIntPtr(proc, char_baseptr, charMaxMP_offset);
  if( not success ) then __err(); return; end;

  PlayerLevel,success = memoryReadIntPtr(proc, char_baseptr, charLevel_offset);
  if( not success ) then __err(); return; end;

  PlayerAngle,success = memoryReadFloatPtr(proc, char_baseptr, charAngle_offset);
  if( not success ) then __err(); return; end;

  PlayerX,success = memoryReadFloat(proc, charXPosition_addr);
  if( not success ) then __err(); return; end;

  PlayerY,success = memoryReadFloat(proc, charYPosition_addr);
  if( not success ) then __err(); return; end;
end

-- Like updateVars(), but only grabs info specific
-- to your target.
function updateTargetInfo()
  function __err()
    error("Error accessing memory from Holic.\n", 0);
  end

  local success;
  Target,success = memoryReadIntPtr(proc, target_ptr, targetID_offset);
  if( not success ) then __err(); return; end;

  TargetHP,success = memoryReadIntPtr(proc, target_ptr, targetHP_offset);
  if( not success ) then __err(); return; end;

  TargetMaxHP,success = memoryReadIntPtr(proc, target_ptr, targetMaxHP_offset);
  if( not success ) then __err(); return; end;
end


-- Returns true if your target is not at full health.
function targetIsHurt()
  if( TargetHP == nil or TargetMaxHP == nil or TargetMaxHP == 0 ) then
    return false; -- something went wrong, assume they aren't hurt.
  end

  return ( TargetHP < TargetMaxHP );
end


function getTargetPtr(proc)
  target_ptr = memoryReadInt(proc, target_baseptr) + target_offset;
  return (target_ptr ~= 0);
end


-- Disable target cycling for a few seconds
-- (part of anti-KS feature)
-- Automatically re-enables itself.
cycleTargetDisabled = false;
function tempDisableCycleTarget()
  cycleTargetDisabled = true;

  registerTimer("tempDisableCycleTarget", secondsToTimer(5),
    function () unregisterTimer("tempDisableCycleTarget")
      cycleTargetDisabled = false
    end );
end

function cycleTarget()
  -- to prevent getting stuck in this function,
  -- we only attempt to cycle targets a few times.

  local cycleAttempts = 0;
  local maxCycleAttempts = 10;

  if( antiKillStealEnabled ) then
    while( true ) do
      yrest(100);
      
      if( not cycleTargetDisabled ) then
        keyboardPress(KEY_TARGET);
        yrest(100);
        
        if( haveTarget() ) then
          if( targetIsHurt() ) then
            printf("Skipping target (anti-KS)\n");
            --clearTarget();
            antiKSTriggerCount = antiKSTriggerCount + 1;
            
            if( antiKSTriggerCount > antiKSTriggerThreshold ) then
              printf("Temporarily disabling auto-target (anti-KS)\n");
              tempDisableCycleTarget();
            end
          else
            antiKSTriggerCount = 0;
            return;
          end
        end

        cycleAttempts = cycleAttempts + 1;
        if( cycleAttempts >= maxCycleAttempts ) then
          return;
        end

      else
        antiKSTriggerCount = 0;
        keyboardPress(KEY_TARGET);
        return;
      end
    end
  else
    keyboardPress(KEY_TARGET);
  end

--[[
 if( antiKillStealEnabled ) then
   if( not targetIsHurt() ) then
     if( not cycleTargetDisabled ) then
       keyboardPress(KEY_TARGET);
     end
   end
  end
]]
end

function clearTarget()

  while( haveTarget() ) do
    keyboardPress(KEY_CLEARTARGET);
    yrest(10);
  end

end

function haveTarget()
  updateTargetInfo();
  --Target = memoryReadIntPtr(proc, target_ptr, targetID_offset);
  return (Target ~= 0)
end

function pauseCallback()
  keyboardRelease(KEY_MOVEFORWARD);
  keyboardRelease(KEY_ROTATELEFT);
  keyboardRelease(KEY_ROTATERIGHT);
  keyboardRelease(KEY_MOVEBACKWARD);

  printf("Paused.\n");
end

function resumeCallback()
  -- re-init movement system.
  initMovementSystem();

  printf("Started.\n");
  printf("Movement system re-initialized.\n");
end

function needHeal()
  if( PlayerMaxHP == 0 ) then return false; end; -- cannot divide by 0, so assume false.

  if( (PlayerHP / PlayerMaxHP) * 100 < HPLowPercent ) then
    return true;
  else
    return false;
  end
end

function needMana()
  if( PlayerMaxMP == 0 ) then return false; end; -- cannot divide by 0, so assume false.

  if( (PlayerMP / PlayerMaxMP) * 100 < MPLowPercent ) then
    return true;
  else
    return false;
  end
end

function useHPPotion()
  printf("Using HP Potion\n");
  yrest(500);
  keyboardHold(key.VK_ALT);
    keyboardPress(KEY_HPPOTION);
  keyboardRelease(key.VK_ALT);
end

function useMPPotion()
  printf("Using MP Potion\n");
  yrest(500);
  keyboardHold(key.VK_ALT);
    keyboardPress(KEY_MPPOTION);
  keyboardRelease(key.VK_ALT);
end

function isPlayerDead()
  return (PlayerHP <= 0);
end


function distance(x1, y1, x2, y2)
  if( x1 == nil or x2 == nil or y1 == nil or y2 == nil ) then
    error("attept to compute distance on nil", 2);
  end

  return math.sqrt( (y2 - y1)*(y2 - y1) + (x2 - x1)*(x2 - x1) );
end

function angleDifference(angle1, angle2)
  if( math.abs(angle2 - angle1) > math.pi ) then
    return (math.pi * 2) - math.abs(angle2 - angle1);
  else
    return math.abs(angle2 - angle1);
  end
end

-- converts atan2-type angles to Holic's angle type.
function fixAngle(angle)
  -- 1.57079633 = 2*PI - 1/2 PI.

  return math.fmod(6.28318531 - angle + 1.57079633, math.pi * 2);
end