-------------------------------------------------------------------
-------------------------------------------------------------------
-- Title:           Holic Bot                                    --
-- Author:          Elverion                                     --
-- Date:            Oct. 13, 2008                                --
-- Requirements:    MicroMacro v0.98                             --
-- License:         Public Domain                                --
-------------------------------------------------------------------
-------------------------------------------------------------------
-- This script is public domain. You may use it in any way you   --
-- find appropriate without restriction. This includes, but is   --
-- not limited to, modification and distribution, whether it be  --
-- commercial or not.                                            --
-------------------------------------------------------------------
-------------------------------------------------------------------


-- a table for holding our waypoints...
waypointQueue = { };

-- current waypoint holders.
current_wpx = PlayerX; current_wpy = PlayerY;

-- Push a new waypoint onto the queue.
function pushWaypoint(wx, wy)
  table.insert(waypointQueue, {x = wx, y = wy});
end

function initMovementSystem()
  updateVars();
  if( movementType == MOVEMENT_WANDER ) then
    WANDER_STARTX = PlayerX;
    WANDER_STARTY = PlayerY;
  end

  current_wpx = PlayerX;
  current_wpy = PlayerY;
end

function stepTowardsWaypoint(targetX, targetY, skipTargetCycle)
  if( skipTargetCycle == nil ) then skipTargetCycle = false; end;

  local minAngleToCorrect = math.rad(15); -- must be within X degrees of target
                                          -- to start walking in that direction.

  local targetAngle = fixAngle(math.atan2(targetY - PlayerY, targetX - PlayerX));

  local dist = distance(PlayerX, PlayerY, targetX, targetY);

  local angleDif = angleDifference(PlayerAngle, targetAngle);

  --local temp = targetAngle - PlayerAngle;

  local lastTimeCheck = os.time();
  local lastDistCheck = dist;

  while( dist > 5 ) do
    __checkSkills(); -- so we can cast out-of-combat skills

    -- check if we have a target BEFORE trying to cycle.
    -- this helps detect aggressive mobs attacking us
    -- before we go and pull more.
    if( haveTarget() ) then
      -- found a target, let's sidetrack and attack.

      keyboardRelease(KEY_ROTATELEFT);
      keyboardRelease(KEY_ROTATERIGHT);
      keyboardRelease(KEY_MOVEFORWARD);

      return true;
    end

    if( not skipTargetCycle ) then
      cycleTarget();
    end


    if( angleDif > minAngleToCorrect ) then
      while( angleDif > minAngleToCorrect ) do

        keyboardRelease(KEY_MOVEFORWARD);
        keyboardRelease(KEY_MOVEBACKWARD);

        if( angleDifference(targetAngle, PlayerAngle + 0.01) < angleDif ) then
          keyboardRelease(KEY_ROTATELEFT);
          keyboardHold(KEY_ROTATERIGHT);
        else
          keyboardRelease(KEY_ROTATERIGHT);
          keyboardHold(KEY_ROTATELEFT);
        end

        yrest(10);
        updateVars();
        targetAngle = fixAngle(math.atan2(targetY - PlayerY, targetX - PlayerX));
        angleDif = angleDifference(PlayerAngle, targetAngle);
      end
      keyboardRelease(KEY_ROTATELEFT);
      keyboardRelease(KEY_ROTATERIGHT);

      -- prevent turning time from being counted
      -- towards "stuck" time.
      lastTimeCheck = os.time();
    end

    -- every few seconds, check if we're stuck.
    if( os.difftime(os.time(), lastTimeCheck) >= 3 ) then
      printf("dist: %d, lastDistCheck: %d\n", dist, lastDistCheck);

      if( math.floor(dist + 1) >= math.floor(lastDistCheck) ) then
        -- looks like we failed to make any sort of improvement...
        printf("We got stuck. Waypoint failed.\n");

        keyboardRelease(KEY_ROTATELEFT);
        keyboardRelease(KEY_ROTATERIGHT);
        keyboardRelease(KEY_MOVEFORWARD);

        yrest(500);

        return false;
      end

      lastDistCheck = dist;
      lastTimeCheck = os.time();
    end

    keyboardHold(KEY_MOVEFORWARD);
    yrest(10);

    targetAngle = fixAngle(math.atan2(targetY - PlayerY, targetX - PlayerX));
    angleDif = angleDifference(PlayerAngle, targetAngle);
    dist = distance(PlayerX, PlayerY, targetX, targetY);


  end

  keyboardRelease(KEY_ROTATELEFT);
  keyboardRelease(KEY_ROTATERIGHT);
  keyboardRelease(KEY_MOVEFORWARD);

  return true; -- successfully reached waypoint.
end


-- If we get stuck, we'll push a set of waypoints that
-- should help us get us unstuck.
function constructUnstuckPath()
  local dist = 20; -- distance for each step.
  local tempX1, tempY1, tempX2, tempY2 = PlayerX, PlayerY, PlayerX, PlayerY;

  local angle1 = PlayerAngle;-- + math.pi;
  local angle2 = PlayerAngle;

  -- construct angles that'll make us walk back and to the side, then forward again (and to the side)
  if( math.random(100) >= 50 ) then
    -- go towards the "left"
    angle1 = angle1 - math.rad(45);
    angle2 = angle2 - math.rad(90);
  else
    -- go towards the "right"
    angle1 = angle1 + math.pi - math.rad(135);
    angle2 = angle2 + math.rad(90);
  end

  -- correct the angles so that they with within -PI -> PI
  angle1 = math.fmod(angle1, math.pi * 2);
  angle2 = math.fmod(angle2, math.pi * 2);
  if( angle1 < 0 ) then angle1 = angle1 + math.pi * 2; end
  if( angle2 < 0 ) then angle2 = angle2 + math.pi * 2; end

  -- convert the angles back to 'standard' angles for use in
  -- sin/cos functions.
  angle1 = angle1 - math.pi + math.pi/2;
  angle2 = angle2 - math.pi + math.pi/2;

  -- calculate x,y coords based on sin/cos w/ angles
  tempX1 = PlayerX + (math.cos(angle1) * dist);
  tempY1 = PlayerY + (math.sin(angle1) * dist);
  tempX2 = PlayerX + (math.cos(angle2) * dist);
  tempY2 = PlayerY + (math.sin(angle2) * dist);

  -- finally, insert the waypoints and see what happens.
  pushWaypoint(tempX1, tempY1);
  pushWaypoint(tempX2, tempY2);
end