-------------------------------------------------------------------
-------------------------------------------------------------------
-- Title:           Holic Bot                                    --
-- Author:          Elverion                                     --
-- Date:            Oct. 13, 2008                                --
-- Requirements:    MicroMacro v0.98                             --
-- License:         Public Domain                                --
-------------------------------------------------------------------
-------------------------------------------------------------------
-- This script is public domain. You may use it in any way you   --
-- find appropriate without restriction. This includes, but is   --
-- not limited to, modification and distribution, whether it be  --
-- commercial or not.                                            --
-------------------------------------------------------------------
-------------------------------------------------------------------

-- a table for storing skills that the player wants to use.
skillList = { };

-- skill requirements...
REQUIREMENT_NONE = 0x0;
REQUIREMENT_HURT = 0x10;
REQUIREMENT_MANALOW = 0x100;
REQUIREMENT_NOCOMBAT = 0x1000;

-- target settings...
TARGET_NONE = 0;
TARGET_SELF = 1;


-- a function for adding a usable skill for this player.
function addSkill(_info, _hotkey)
  if( _info == nil ) then
    error("Invalid skill passed to addSkill()\n", 2);
  end

  local tmp = Skill.create(_info, _hotkey);

  table.insert(skillList, tmp);
end



-- shortcut for creating a skillinfo table.
function skillInfo(_name, _cooldown, _casttime, _mana, _requirement, _targettype)
  if( _name == nil ) then _name = "Unknown Skill"; end;
  if( _cooldown == nil ) then _cooldown = 30; end;
  if( _casttime == nil ) then _casttime = 3; end;
  if( _mana == nil ) then _mana = 0; end;
  if( _requirement == nil ) then _requirement = REQUIREMENT_NONE; end;
  if( _targettype == nil ) then _targettype = TARGET_NONE; end;

  return { name = _name, cooldown = _cooldown, casttime = _casttime,
  mana = _mana, requirement = _requirement, targettype = _targettype};
end



-- skill class
Skill = { };
Skill.__index = Skill;

function Skill.create(_info, _hotkey)
  local tmp = {};
  setmetatable(tmp, Skill);
  tmp.info = _info;
  tmp.hotkey = _hotkey;
  tmp.lastUse = 0;

  return tmp;
end

function Skill:ready()
  if( self.lastUse == 0 ) then return true; end;

  return ( os.difftime(os.time(), self.lastUse) > self.info.cooldown );
end

function Skill:canUse()
  if( not self:ready() ) then return false; end;

  if( PlayerMP < self.info.mana ) then return false; end;

  if( bit.AND(self.info.requirement, REQUIREMENT_HURT) and (PlayerHP/PlayerMaxHP) > 0.75 ) then return false; end;

  if( bit.AND(self.info.requirement, REQUIREMENT_NOCOMBAT) and PlayerInCombat ) then return false; end;
  if( not(bit.AND(self.info.requirement, REQUIREMENT_NOCOMBAT)) and (not PlayerInCombat) ) then return false; end;

  if( bit.AND(self.info.requirement, REQUIREMENT_MANALOW) and (PlayerMP/PlayerMaxMP) > 0.75 ) then return false; end;


  return true;
end

function Skill:cast()
  printf("Casting skill \'%s\'\n", self.info.name);

  keyboardPress(self.hotkey);

  if( self.info.targettype == TARGET_SELF and haveTarget() ) then
    wx,wy,ww,wh = windowRect(win);

    -- click on our face icon in the top-left corner.
    printf("Targetting self...\n");
    mx, my = mouseGetPos();
    mouseSet(wx + 48, wy + 48);
    yrest(100);
    mouseLClick();
    yrest(100);
    mouseSet(mx, my);
  end

  yrest(self.info.casttime * 1000);
  self.lastUse = os.time();
end


-- include class-specific skill information
include("database/skills_priest.lua");
include("database/skills_monk.lua");
include("database/skills_warrior.lua");